/*
 * Copyright (C) 2012, H&D Wireless AB All rights reserved.
 *
 * The license to use this software in whole and in part and to
 * redistribute it in any form follows with the WiFi HW module from H&D
 * Wireless and is granted under the following restrictions:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * 3. The name of H&D Wireless AB may not be used to endorse or promote
 *    products derived from this software without specific prior written
 *    permission.
 *
 * 4. The software may only be used together with hardware from H&D
 *    Wireless all other use is prohibited.
 *
 * 5. The license to use and redistribute the software is granted
 *    together with the purchase of a hardware platform on a one to one
 *    basis.
 *
 * 6. The binary code may not be reversed engineered or by other means
 *    copied to circumvent this license.
 * 
 * THIS SOFTWARE IS PROVIDED BY H&D WIRELESS AB ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE EXPRESSLY AND SPECIFICALLY DISCLAIMED. IN NO EVENT
 * SHALL HD WIRELESS AB BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * For more information regarding this software license Contact H&D
 * Wireless AB (support@hd-wireless.se).
 *
 */


#define OWL_TRACE OWL_TRACE_INFO
#define OWL_FILE "owl_thread"

#include <owl/core/owl_defs.h>
#include <owl/core/owl_debug.h>
#include <owl/core/owl_err.h>
#include <owl/core/owl_thread.h>

int
owlmutex_init(owlmutex_t *mutex)
{
        pthread_mutex_init(mutex, NULL);
	return 0;
}

void
owlmutex_destroy(owlmutex_t *mutex)
{
        owl_unused(mutex);
        return;
}

int
owlmutex_lock(owlmutex_t *mutex)
{
	int err;
	if ((err = pthread_mutex_lock(mutex))) {
		owl_err("fail to lock mutex, err:%d", err);
	}

        return err;
}

void
owlmutex_unlock(owlmutex_t *mutex)
{
	int err;
	if ((err = pthread_mutex_unlock(mutex))) {
		owl_err("fail to unlock mutex, err:%d", err);
		owl_assert(0);
	}
}

int
owlsem_init(owlsem_t *sem, int count)
{
	int err;
	if ((err = sem_init(sem, 0, count))) {
		owl_err("fail to create sem, err:%d", err);
		return OWL_FAILURE;
	}
	
	return 0;
}


void
owlsem_destroy(owlsem_t *sem)
{
	int err;
	if ((err = sem_destroy(sem))) {
		owl_err("fail to destroy sem, err:%d", err);
		owl_assert(0);
	}
}


void
owlsem_post(owlsem_t *sem)
{
	int err;
	if ((err = sem_post(sem))) {
		owl_err("fail to post sem, err:%d", err);
		owl_assert(0);
	}
}

int
owlsem_wait(owlsem_t *sem)
{
	int err;
	if ((err = sem_wait(sem))) {
		owl_err("fail to wait sem, err:%d", err);
		return OWL_FAILURE;
	}
	
	return 0;
}

void
owlsem_post_isr(owlsem_t *sem)
{
	sem_post(sem);
}

int
owlsem_wait_tmo(owlsem_t *sem, unsigned int ms)
{
        struct timespec tm;
        tm.tv_sec = ms / 1000;
        tm.tv_nsec = (ms % 1000) * 1000 * 1000;
        
        return sem_timedwait(sem, &tm);
}

owlthread_t
owlthread_create(void (*f)(void *arg), void *arg)
{
        pthread_t thread;
        int err;
        
        if ((err = pthread_create(
                     &thread, NULL, (void *(*)(void *)) f, arg))) {
                owl_err("fail to create thread, err:%d", err);
                return 0;
        }

        return thread;
}

void
owlthread_join(owlthread_t thread, int *status)
{
        pthread_join(thread, (void **) status);
}

void
owlthread_exit(int status)
{
        pthread_exit((void *) status);
}

void
owlthread_yield(void)
{
	int err;

        err = sched_yield();
        owl_assert(!err);
}

void
owlthread_sleep(int ms)
{
        usleep(ms * 1000);
}

owlthread_t
owlthread_self(void)
{
        return pthread_self();
}
          
