/*
 * Copyright (C) 2012, H&D Wireless AB All rights reserved.
 *
 * The license to use this software in whole and in part and to
 * redistribute it in any form follows with the WiFi HW module from H&D
 * Wireless and is granted under the following restrictions:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * 3. The name of H&D Wireless AB may not be used to endorse or promote
 *    products derived from this software without specific prior written
 *    permission.
 *
 * 4. The software may only be used together with hardware from H&D
 *    Wireless all other use is prohibited.
 *
 * 5. The license to use and redistribute the software is granted
 *    together with the purchase of a hardware platform on a one to one
 *    basis.
 *
 * 6. The binary code may not be reversed engineered or by other means
 *    copied to circumvent this license.
 * 
 * THIS SOFTWARE IS PROVIDED BY H&D WIRELESS AB ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE EXPRESSLY AND SPECIFICALLY DISCLAIMED. IN NO EVENT
 * SHALL HD WIRELESS AB BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * For more information regarding this software license Contact H&D
 * Wireless AB (support@hd-wireless.se).
 *
 */

/**
 * @file owl_gpio.c
 *
 * @brief AVR8-port of the owl_gpio interface.
 *
 * GPIO-pins are identified as their offset from the start of PORTA in
 * the peripheral module address map. The can be defined by the
 * IOPORT_CREATE_PIN() macro from ioport.h or by manually
 * adding the pin number to the port offset :
 *
 * Example :  
 * @code
 * #define FOO_PORT PORT_PORTF
 * #define FOO_PORT_PIN 3
 * #define FOO_PIN (FOO_PORT + FOO_PIN)
 *
 * owlgpio_set(FOO_PIN);
 * @endcode
 *
 * See owl_gpio.h for API details.
 */

#define OWL_TRACE OWL_TRACE_INFO

#include <avr/io.h>
#include <util/delay_basic.h>

#define F_CPU 32000000UL
#include <util/delay.h>
#include <ioport.h>
#include <board.h>

#include <board.h>
#include <sysclk.h>

#include <owl/core/owl_debug.h>
#include <owl/core/owl_err.h>

#include <owl/drivers/owl_gpio.h>

int 
owlgpio_init(void) 
{
        /* No init needed on this platform */
        return 0;
}

int 
owlgpio_cfg(int pin, int flags) 
{
        if (flags & OWL_GPIO_INPUT)
                ioport_configure_pin(pin, IOPORT_DIR_INPUT);
        else if (flags & OWL_GPIO_OUTPUT)
                ioport_configure_pin(pin, IOPORT_DIR_OUTPUT);
        return 0;
}

int 
owlgpio_is_high(int pin)
{
        return ioport_pin_is_high(pin);
}

void 
owlgpio_set(int pin)
{
        ioport_set_pin_high(pin);
}

void 
owlgpio_clr(int pin)
{
        ioport_set_pin_low(pin);
}

