/*
 * Copyright (C) 2012, H&D Wireless AB All rights reserved.
 *
 * The license to use this software in whole and in part and to
 * redistribute it in any form follows with the WiFi HW module from H&D
 * Wireless and is granted under the following restrictions:
 * 
 * 1. Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in the
 * documentation and/or other materials provided with the distribution.
 *
 * 3. The name of H&D Wireless AB may not be used to endorse or promote
 * products derived from this software without specific prior written
 * permission.
 *
 * 4. The software may only be used together with hardware from H&D
 * Wireless all other use is prohibited.
 *
 * 5. The license to use and redistribute the software is granted
 * together with the purchase of a hardware platform on a one to one
 * basis.
 *
 * 6. The binary code may not be reversed engineered or by other means
 * copied to circumvent this license.
 * 
 * THIS SOFTWARE IS PROVIDED BY H&D WIRELESS AB ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE EXPRESSLY AND SPECIFICALLY DISCLAIMED. IN NO EVENT
 * SHALL HD WIRELESS AB BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * For more information regarding this software license Contact H&D
 * Wireless AB (support@hd-wireless.se).
 *
 */

#define OWL_TRACE OWL_TRACE_DBG

#include <owl/core/owl_defs.h>
#include <owl/core/owl_debug.h>
#include <owl/core/owl_err.h>
#include <owl/core/owl_os.h>
#include <owl/drivers/owl_board.h>

#include <util/eeprom.h>

#include <nvm.h>

static int eeprom_erase(struct mtd_dev *dev, uint32_t addr, uint32_t len);
static int eeprom_read(struct mtd_dev *dev, uint32_t addr, char *buf, uint32_t len);
static int eeprom_write(struct mtd_dev *dev, uint32_t addr,
                        const char *buf, uint32_t len);
static void eeprom_suspend(struct mtd_dev *dev);
static void eeprom_resume(struct mtd_dev *dev);

struct mtd_dev *
eeprom_create(void)
{
        struct mtd_dev *dev;

        owl_info("enter");
        
        if ((dev = owl_os_zalloc(sizeof(*dev))) == NULL) {
                owl_err("could not alloc dev");
                return NULL;
        }

        dev->read = eeprom_read;
        dev->write = eeprom_write;
        dev->erase = eeprom_erase;
        dev->suspend = eeprom_suspend;
        dev->resume = eeprom_resume;
        dev->type = MTD_TYPE_EEPROM;
        dev->flags = MTD_WRITEABLE;
        dev->writesize = EEPROM_SIZE;
        dev->erasesize[0] = 1;
        dev->erasesize[1] = EEPROM_PAGE_SIZE;
        dev->erasesize[2] = EEPROM_SIZE;
        dev->erasesize[3] = 0;
        dev->size = EEPROM_SIZE;
        dev->name = "eeprom";

        owl_info("name:%s size:%d writesize:%d erasesize:%d,%d,%d,%d",
                 dev->name, dev->size, dev->writesize, dev->erasesize[0],
                 dev->erasesize[1], dev->erasesize[2], dev->erasesize[3]);
        
        return dev;
}


static int
eeprom_erase(struct mtd_dev *dev, uint32_t addr, uint32_t len)
{
       
        owl_dbg("dev:%p addr:0x%x len:0x%x", dev, addr, len);
        return 0;
}


static int
eeprom_read(struct mtd_dev *dev, uint32_t addr, char *buf, uint32_t len)
{       
        owl_dbg("dev:%p addr:0x%x buf:%p len:0x%x", dev, addr, buf, len);
        nvm_eeprom_read_buffer(addr, buf, len);
        return len;
}

static int
eeprom_write(struct mtd_dev *dev, uint32_t addr, const char *buf, uint32_t len)
{
        owl_dbg("dev:%p addr:0x%x buf:%p len:0x%x", dev, addr, buf, len);

        if (dev == NULL) {
                owl_err("dev was NULL");
                return OWL_ERR_PARAM;
        }

        if (len <= 0)
                return 0;

        if (addr + len > dev->size) {
                owl_err("trying to write blocks that are out of range");
                return OWL_ERR_PARAM;
        }
        
        nvm_eeprom_erase_and_write_buffer(addr, buf, len);
        return len;
}


static void
eeprom_suspend(struct mtd_dev *dev)
{
        return;
}


static void
eeprom_resume(struct mtd_dev *dev)
{
        return;
}
