/*
 * Copyright (C) 2012, H&D Wireless AB All rights reserved.
 *
 * The license to use this software in whole and in part and to
 * redistribute it in any form follows with the WiFi HW module from H&D
 * Wireless and is granted under the following restrictions:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * 3. The name of H&D Wireless AB may not be used to endorse or promote
 *    products derived from this software without specific prior written
 *    permission.
 *
 * 4. The software may only be used together with hardware from H&D
 *    Wireless all other use is prohibited.
 *
 * 5. The license to use and redistribute the software is granted
 *    together with the purchase of a hardware platform on a one to one
 *    basis.
 *
 * 6. The binary code may not be reversed engineered or by other means
 *    copied to circumvent this license.
 * 
 * THIS SOFTWARE IS PROVIDED BY H&D WIRELESS AB ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE EXPRESSLY AND SPECIFICALLY DISCLAIMED. IN NO EVENT
 * SHALL HD WIRELESS AB BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * For more information regarding this software license Contact H&D
 * Wireless AB (support@hd-wireless.se).
 *
 */

#define OWL_TRACE OWL_TRACE_ERR

#include <owl/core/owl_debug.h>
#include <owl/core/owl_defs.h>
#include <owl/core/owl_err.h>
#include <owl/drivers/owl_board.h>
#include <owl/drivers/owl_gpio.h>
#include <owl/components/spb800p.h>

int
spb800p_resume(void)
{
        /* If spb800p is connected with SPI-interface, make sure that we
           drive the SPI-IRQ low to indicate that host wants to use SPI mode */
#if defined(OWL_BOARD_SPB800P_SPI_IRQ_PIN)
        owl_dbg("conf irq pin, initially as output to put SPB800P in SPI mode");
        owlgpio_cfg(OWL_BOARD_SPB800P_SPI_IRQ_PIN, OWL_GPIO_OUTPUT);
        owlgpio_clr(OWL_BOARD_SPB800P_SPI_IRQ_PIN);
#endif

        /* Host attention should be an input */
#if defined(OWL_BOARD_SPB800P_HOST_ATTENTION_PIN)
        owl_dbg("conf host attention pin as input");
        owlgpio_cfg(OWL_BOARD_SPB800P_HOST_ATTENTION_PIN, OWL_GPIO_INPUT);
#endif

        /* Wakeup should be an output. It's active low, so clear it so the
           spb800p wakes up during startup XXX */
#if defined(OWL_BOARD_SPB800P_WAKEUP_PIN)
        owl_dbg("conf wakeup pin to wakeup SPB800P");
        owlgpio_cfg(OWL_BOARD_SPB800P_WAKEUP_PIN, OWL_GPIO_OUTPUT);
        owlgpio_clr(OWL_BOARD_SPB800P_WAKEUP_PIN);
#endif

        /* Now we plug the power to SPB800 */
#if defined (OWL_BOARD_SPB800P_SHUTDOWN_PIN)
        owl_dbg("conf shutdown pin to power on SPB800P");
        owlgpio_cfg(OWL_BOARD_SPB800P_SHUTDOWN_PIN, OWL_GPIO_OUTPUT);
        owlgpio_set(OWL_BOARD_SPB800P_SHUTDOWN_PIN);
        owlboard_mdelay(500);
        owlgpio_clr(OWL_BOARD_SPB800P_SHUTDOWN_PIN);
#endif
        /* Wait for wifi to start */
        owl_dbg("wait for SPB800P to start");
        owlboard_mdelay(1300);

        /* Detect startup in SPI mode. In UART mode, the dots will indicate
           complete startup (handled in wlp_init()) */
#if defined(OWL_BOARD_SPB800P_SPI_IRQ_PIN)
        /* Reconfigure pin as input to detect SPB800P startup, we could just
           count to 10 as well if this pin is unconnected on the host... */
        owlgpio_cfg(OWL_BOARD_SPB800P_SPI_IRQ_PIN,
                    OWL_GPIO_INPUT | OWL_GPIO_PULL_UP);
        owl_info("wait for IRQ to go low");
        while (owlgpio_is_high(OWL_BOARD_SPB800P_SPI_IRQ_PIN));
        owl_info("wait for IRQ to go high");
        while (owlgpio_is_low(OWL_BOARD_SPB800P_SPI_IRQ_PIN));
        owl_info("device started");
#endif
        
        return 0;
}

int
spb800p_suspend(void)
{
#if defined (OWL_BOARD_SPB800P_SHUTDOWN_PIN)
        owl_dbg("conf shutdown pin to power off SPB800P");
        owlgpio_cfg(OWL_BOARD_SPB800P_SHUTDOWN_PIN, OWL_GPIO_OUTPUT);
        owlgpio_set(OWL_BOARD_SPB800P_SHUTDOWN_PIN);
#endif
        return 0;
}
