/*
 * Copyright (C) 2012, H&D Wireless AB All rights reserved.
 *
 * The license to use this software in whole and in part and to
 * redistribute it in any form follows with the WiFi HW module from H&D
 * Wireless and is granted under the following restrictions:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * 3. The name of H&D Wireless AB may not be used to endorse or promote
 *    products derived from this software without specific prior written
 *    permission.
 *
 * 4. The software may only be used together with hardware from H&D
 *    Wireless all other use is prohibited.
 *
 * 5. The license to use and redistribute the software is granted
 *    together with the purchase of a hardware platform on a one to one
 *    basis.
 *
 * 6. The binary code may not be reversed engineered or by other means
 *    copied to circumvent this license.
 * 
 * THIS SOFTWARE IS PROVIDED BY H&D WIRELESS AB ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE EXPRESSLY AND SPECIFICALLY DISCLAIMED. IN NO EVENT
 * SHALL HD WIRELESS AB BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * For more information regarding this software license Contact H&D
 * Wireless AB (support@hd-wireless.se).
 *
 */

#ifndef WLP_TRAMPOLINE_H
#define WLP_TRAMPOLINE_H

#if WLP_TRAMPOLINE

#if WLP_STACK_OFFLOAD
int _wlp_init(int interface,
              int (*readf)(void *ctx, void *data, int len),
              int (*writef)(void *ctx, const void *data, int len),
              void (*selectf)(void *ctx, int select),
              void *ctx);
#else
int _wlp_init(int (*bus_f)(void *ctx, const void *tx, void *rx, int len),
              void (*select_f)(void *ctx, int select),
              void (*irqen_f)(void *ctx, int enable),
              void (*firmware_f)(void *ctx, int enable),
              void *ctx,
              int flags);
#endif /* WLP_STACK_OFFLOAD XXX */

void _wlp_shutdown(void);
void _wlp_set_yield_cb(void (*yieldf)(void *ctx), void *ctx);
void _wlp_irq(void);
int _wlp_set_baudrate(uint32_t baudrate, uint8_t rtscts);
#if WLP_STACK_OFFLOAD
int _wlp_fw_upgrade(uint8_t method, struct ip_addr *addr, uint16_t port,
                    const char *path);
int _wlp_set_fw_upgrade_cb(void (*status_cb)(void *ctx, uint8_t success),
                           void *ctx);
#endif
int _wlp_get_fw_version(uint32_t *code, char *build, int build_len,
                        char *hw_rev, int hw_rev_len);
int _wlp_get_fw_caps(uint8_t *cap_flags);
int _wlp_scan(struct wlp_ssid *ssid,
             void (*scan_cb)(void *ctx, struct wlp_network *net), void *ctx);
void _wlp_set_link_cb(void (*link_cb)(void *ctx, int link), void *ctx);
int _wlp_linkup(struct wlp_ssid *ssid, const char *key,
                enum wlp_auth_mode auth);
int _wlp_linkdown(void);
int _wlp_set_wps(int enable);
int _wlp_get_hwaddr(struct wlp_mac_addr *hwaddr);
int _wlp_get_network(struct wlp_network *net);
int _wlp_get_link_context(void (*linkctx_cb)(void *ctx, void *linkctx, int len),
                         void *ctx);
int _wlp_set_link_context(void *linkctx);
#if WLP_STACK_OFFLOAD
void _wlp_set_ipaddr_cb(void (*addr_cb)(void *ctx, const struct ip_addr *addr),
                       void *ctx);
int _wlp_set_ipaddr(const struct ip_addr *ip, const struct ip_addr *netmask,
                   const struct ip_addr *gw, const struct ip_addr *dns);
int _wlp_get_ipaddr(struct ip_addr *ip, struct ip_addr *netmask,
                   struct ip_addr *gw, struct ip_addr *dns);
int _wlp_set_dhcp(int enable);
int _wlp_get_dhcp(int *enabled);
int _wlp_set_dhcpd(int enable);
int _wlp_socket(int type, int proto);
int _wlp_set_conn_cb(int sockid,
                    void (*conn_cb)(void *ctx, int sockid, int connected),
                    void *ctx);
int _wlp_set_recv_cb(int sockid,
                    void (*recv_cb)(void *ctx, int sockid, int len),
                    void *ctx);
int _wlp_set_sent_cb(int sockid,
                    void (*sent_cb)(void *ctx, int sockid, int len),
                    void *ctx);
int _wlp_bind(int sockid, const struct ip_addr *addr, uint16_t port);
int _wlp_listen(int sockid, int backlog,
               void (*listen_cb)(void *ctx, int sockid),
               void *ctx);
int _wlp_connect(int sockid, const struct ip_addr *ip, uint16_t port);
int _wlp_accept(int sockid);
int _wlp_close(int sockid);
int _wlp_recv(int sockid, char *buf, int16_t len);
int _wlp_send(int sockid, const char *buf, int16_t len);
int _wlp_sendto(int sockid, const char *buf, int16_t len,
               const struct ip_addr *ip, uint16_t port);
int _wlp_get_peeraddr(int sockid, struct ip_addr *peer);
int _wlp_get_hostbyname(const char *host,
                       void (*lookup_cb)(void *ctx, const struct ip_addr *ip),
                       void *ctx);
#else
void _wlp_set_recv_cb(int (*recv_cb)(void *ctx, int len), void *ctx);
int _wlp_process_tx(void *ethhdr, int len, int totlen, void *wlxhdr);
int _wlp_tx(void *data, int len);
int _wlp_rx(void *data, int len);
int _wlp_process_rx(void *data, int len, void **stripped, int *stripped_len);
#endif /* WLP_STACK_OFFLOAD */
int _wlp_poll(void);
int _wlp_set_mode(uint8_t mode);
int _wlp_get_mode(uint8_t *mode);
int _wlp_set_channel(uint8_t channel,
                     void (*channel_cb)(void *ctx, int status), void *ctx);
int _wlp_conf_ap(int hidden, int beacon_intvl, int inact_tmo,
                const char *country);
int _wlp_set_wakeup_cb(void (*wakeupf)(void *ctx, int enable),
                      void *ctx);
int _wlp_set_ps(int enable);
int _wlp_conf_ps(uint8_t use_ps_poll,
                uint32_t ps_traffic_timeout,
                uint32_t ps_delay,
                uint8_t rx_all_dtim,
                uint16_t listen_interval);
int _wlp_set_heartbeat(uint32_t ms_period);
int _wlp_set_led(int enable);
int _wlp_set_poll_period(uint32_t ms_period);

# else /* !WLP_TRAMPOLINE */

#define wlp_init                _wlp_init
#define wlp_shutdown            _wlp_shutdown
#define wlp_set_yield_cb        _wlp_set_yield_cb
#define wlp_irq                 _wlp_irq
#define wlp_set_baudrate        _wlp_set_baudrate
#define wlp_reset               _wlp_reset
#define wlp_fw_upgrade          _wlp_fw_upgrade
#define wlp_set_fw_upgrade_cb   _wlp_set_fw_upgrade_cb
#define wlp_get_fw_version      _wlp_get_fw_version
#define wlp_get_fw_caps         _wlp_get_fw_caps
#define wlp_scan                _wlp_scan
#define wlp_set_link_cb         _wlp_set_link_cb
#define wlp_linkup              _wlp_linkup
#define wlp_linkdown            _wlp_linkdown
#define wlp_set_wps             _wlp_set_wps
#define wlp_get_hwaddr          _wlp_get_hwaddr
#define wlp_get_network         _wlp_get_network
#define wlp_get_link_context    _wlp_get_link_context
#define wlp_set_link_context    _wlp_set_link_context
#define wlp_set_ipaddr_cb       _wlp_set_ipaddr_cb
#define wlp_set_ipaddr          _wlp_set_ipaddr
#define wlp_get_ipaddr          _wlp_get_ipaddr
#define wlp_set_dhcp            _wlp_set_dhcp
#define wlp_get_dhcp            _wlp_get_dhcp
#define wlp_set_dhcpd           _wlp_set_dhcpd
#define wlp_socket              _wlp_socket
#define wlp_set_conn_cb         _wlp_set_conn_cb
#define wlp_set_recv_cb         _wlp_set_recv_cb
#define wlp_set_sent_cb         _wlp_set_sent_cb
#define wlp_bind                _wlp_bind
#define wlp_connect             _wlp_connect
#define wlp_listen              _wlp_listen
#define wlp_accept              _wlp_accept
#define wlp_close               _wlp_close
#define wlp_recv                _wlp_recv
#define wlp_send                _wlp_send
#define wlp_sendto              _wlp_sendto
#define wlp_get_peeraddr        _wlp_get_peeraddr
#define wlp_get_hostbyname      _wlp_get_hostbyname
#define wlp_set_recv_cb         _wlp_set_recv_cb
#define wlp_process_tx          _wlp_process_tx
#define wlp_tx                  _wlp_tx
#define wlp_rx                  _wlp_rx
#define wlp_process_rx          _wlp_process_rx
#define wlp_poll                _wlp_poll
#define wlp_set_mode            _wlp_set_mode
#define wlp_get_mode            _wlp_get_mode
#define wlp_set_channel         _wlp_set_channel
#define wlp_conf_ap             _wlp_conf_ap
#define wlp_conf_ps             _wlp_conf_ps
#define wlp_set_wakeup_cb       _wlp_set_wakeup_cb
#define wlp_set_ps              _wlp_set_ps
#define wlp_set_heartbeat       _wlp_set_heartbeat
#define wlp_set_led             _wlp_set_led
#define wlp_set_poll_period     _wlp_set_poll_period

#endif /* WLP_TRAMPOLINE */


#endif /* WLP_TRAMPOLINE_H */
